selenium_wrapper_web_driver_not_found_error = "Web Driver not found"

selenium_wrapper_opera_path_error = "Opera need executable path"

selenium_wrapper_set_options_error = "only accept dict type"
selenium_wrapper_set_argument_error = "only accept str type"

executor_data_error = "executor receive wrong data"
executor_list_error = "executor receive wrong data list is none or wrong type"


cant_find_json_error = "can't find json"
cant_save_json_error = "can't save json"
cant_reformat_json_error = "can't reformat json is type right?"
cant_find_element_in_json_error = "can't find element in json"
json_type_error = "json type error"
wrong_json_data_error = "can't parser json"
